/********************************************
 Ultrasonic Digital Tape Measure
 by: Jim Lindblom
 SparkFun Electronics
 
 This is a simple sketch designed to show off 
 our Maxbotix Ultrasonic sensors. Readings will
 be taken, averaged and displayed, in inches, on
 the 7-segment display.
 
 Note: using software serial library to avoid 
 hooking up the 7-segment serial display to the
 hardware serial pins and accidentally sending it
 weird characters during Arduino programming.
 
 Hardware setup:
 'AN' of ultrasonic to 'A0' of Arduino
 '+5' of ultrasonic to '5V' of Arduino
 'GND' of ultrasonic to 'GND' of Arduino
 
 'RX' of Serial 7-Segment Display to 'D3' of Arduino
 'VCC' of Serial 7-Segment Display to '5V' of Arduino
 'GND' of Serial 7-Segment Display to 'GND' of Arduino
 **********************************************/
#include <SoftwareSerial.h>

#define SensorPin 0  // Ultrasonic analog output
#define TXPin 3
#define RXPin 2
#define AVG_COUNT 10  // Don't set this too high, 10 seems pretty good

unsigned int inches[AVG_COUNT];
unsigned int average;
unsigned char dig0, dig1, dig2, dig3;

SoftwareSerial Serial7Seg = SoftwareSerial(RXPin, TXPin);

void setup()
{
  pinMode(RXPin, INPUT);
  pinMode(TXPin, OUTPUT);
  
  Serial7Seg.begin(9600);
  
  Serial7Seg.print("v");  // 'reset' display command
  Serial7Seg.print("-");  // display startup message "-Hi-" to verify function of display
  Serial7Seg.print("H");
  Serial7Seg.print("I");
  Serial7Seg.print("-");
}

void loop()
{
  average = 0;
  
  for (int i = 0; i < AVG_COUNT; i++)
  {
    inches[i] = analogRead(SensorPin);
    inches[i] /= 2;  // convert to inches
    delay(50);  // update rate is 20Hz
  }
  for (int i = 0; i<AVG_COUNT; i++)
    average += inches[i];
  average /= AVG_COUNT;

  // delay for display readability
  // you may need to mess with this if you change AVG_COUNT
  if (AVG_COUNT <= 50)
    delay(100-AVG_COUNT/2);
  
  // Separate average into 4 digits
  dig0 = average/1000;
  dig1 = average/100 - dig0*10;
  dig2 = average/10 - dig0*100 - dig1*10;
  dig3 = average - dig0*1000 - dig1*100 - dig2*10;
  
  Serial7Seg.print("v");  // 'reset' display to first position
  Serial7Seg.print(dig0);  // Send first digit to display
  Serial7Seg.print(dig1);
  Serial7Seg.print(dig2);
  Serial7Seg.print(dig3);  // Send last digit to display
}

